from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.container_create import ContainerCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainersBulkCreateRequest")


@attr.s(auto_attribs=True, repr=False)
class ContainersBulkCreateRequest:
    """  """

    _containers: List[ContainerCreate]

    def __repr__(self):
        fields = []
        fields.append("containers={}".format(repr(self._containers)))
        return "ContainersBulkCreateRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        containers = []
        for containers_item_data in self._containers:
            containers_item = containers_item_data.to_dict()

            containers.append(containers_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if containers is not UNSET:
            field_dict["containers"] = containers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_containers() -> List[ContainerCreate]:
            containers = []
            _containers = d.pop("containers")
            for containers_item_data in _containers:
                containers_item = ContainerCreate.from_dict(containers_item_data, strict=False)

                containers.append(containers_item)

            return containers

        try:
            containers = get_containers()
        except KeyError:
            if strict:
                raise
            containers = cast(List[ContainerCreate], UNSET)

        containers_bulk_create_request = cls(
            containers=containers,
        )

        return containers_bulk_create_request

    @property
    def containers(self) -> List[ContainerCreate]:
        if isinstance(self._containers, Unset):
            raise NotPresentError(self, "containers")
        return self._containers

    @containers.setter
    def containers(self, value: List[ContainerCreate]) -> None:
        self._containers = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.nucleotide_alignment_base_algorithm import NucleotideAlignmentBaseAlgorithm
from ..models.nucleotide_alignment_base_files_item import NucleotideAlignmentBaseFilesItem
from ..models.nucleotide_alignment_file import NucleotideAlignmentFile
from ..types import UNSET, Unset

T = TypeVar("T", bound="NucleotideAlignmentBase")


@attr.s(auto_attribs=True, repr=False)
class NucleotideAlignmentBase:
    """  """

    _algorithm: NucleotideAlignmentBaseAlgorithm
    _files: List[Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]]
    _name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("algorithm={}".format(repr(self._algorithm)))
        fields.append("files={}".format(repr(self._files)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "NucleotideAlignmentBase({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        algorithm = self._algorithm.value

        files = []
        for files_item_data in self._files:
            if isinstance(files_item_data, UnknownType):
                files_item = files_item_data.value
            elif isinstance(files_item_data, NucleotideAlignmentBaseFilesItem):
                files_item = files_item_data.to_dict()

            else:
                files_item = files_item_data.to_dict()

            files.append(files_item)

        name = self._name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if algorithm is not UNSET:
            field_dict["algorithm"] = algorithm
        if files is not UNSET:
            field_dict["files"] = files
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_algorithm() -> NucleotideAlignmentBaseAlgorithm:
            _algorithm = d.pop("algorithm")
            try:
                algorithm = NucleotideAlignmentBaseAlgorithm(_algorithm)
            except ValueError:
                algorithm = NucleotideAlignmentBaseAlgorithm.of_unknown(_algorithm)

            return algorithm

        try:
            algorithm = get_algorithm()
        except KeyError:
            if strict:
                raise
            algorithm = cast(NucleotideAlignmentBaseAlgorithm, UNSET)

        def get_files() -> List[
            Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]
        ]:
            files = []
            _files = d.pop("files")
            for files_item_data in _files:

                def _parse_files_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]:
                    files_item: Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        files_item = NucleotideAlignmentBaseFilesItem.from_dict(data, strict=True)

                        return files_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        files_item = NucleotideAlignmentFile.from_dict(data, strict=True)

                        return files_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                files_item = _parse_files_item(files_item_data)

                files.append(files_item)

            return files

        try:
            files = get_files()
        except KeyError:
            if strict:
                raise
            files = cast(
                List[Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]], UNSET
            )

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        nucleotide_alignment_base = cls(
            algorithm=algorithm,
            files=files,
            name=name,
        )

        nucleotide_alignment_base.additional_properties = d
        return nucleotide_alignment_base

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def algorithm(self) -> NucleotideAlignmentBaseAlgorithm:
        if isinstance(self._algorithm, Unset):
            raise NotPresentError(self, "algorithm")
        return self._algorithm

    @algorithm.setter
    def algorithm(self, value: NucleotideAlignmentBaseAlgorithm) -> None:
        self._algorithm = value

    @property
    def files(self) -> List[Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]]:
        if isinstance(self._files, Unset):
            raise NotPresentError(self, "files")
        return self._files

    @files.setter
    def files(
        self, value: List[Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]]
    ) -> None:
        self._files = value

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="DropdownOptionsUnarchive")


@attr.s(auto_attribs=True, repr=False)
class DropdownOptionsUnarchive:
    """  """

    _dropdown_option_ids: Union[Unset, List[str]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("dropdown_option_ids={}".format(repr(self._dropdown_option_ids)))
        return "DropdownOptionsUnarchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        dropdown_option_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._dropdown_option_ids, Unset):
            dropdown_option_ids = self._dropdown_option_ids

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if dropdown_option_ids is not UNSET:
            field_dict["dropdownOptionIds"] = dropdown_option_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_dropdown_option_ids() -> Union[Unset, List[str]]:
            dropdown_option_ids = cast(List[str], d.pop("dropdownOptionIds"))

            return dropdown_option_ids

        try:
            dropdown_option_ids = get_dropdown_option_ids()
        except KeyError:
            if strict:
                raise
            dropdown_option_ids = cast(Union[Unset, List[str]], UNSET)

        dropdown_options_unarchive = cls(
            dropdown_option_ids=dropdown_option_ids,
        )

        return dropdown_options_unarchive

    @property
    def dropdown_option_ids(self) -> List[str]:
        """ Array of dropdown option IDs """
        if isinstance(self._dropdown_option_ids, Unset):
            raise NotPresentError(self, "dropdown_option_ids")
        return self._dropdown_option_ids

    @dropdown_option_ids.setter
    def dropdown_option_ids(self, value: List[str]) -> None:
        self._dropdown_option_ids = value

    @dropdown_option_ids.deleter
    def dropdown_option_ids(self) -> None:
        self._dropdown_option_ids = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.dropdown_dependency import DropdownDependency
from ..models.entity_schema_dependency import EntitySchemaDependency
from ..models.manifest_array_config_type import ManifestArrayConfigType
from ..models.manifest_scalar_config import ManifestScalarConfig
from ..models.resource_dependency import ResourceDependency
from ..models.schema_dependency import SchemaDependency
from ..models.workflow_task_schema_dependency import WorkflowTaskSchemaDependency
from ..types import UNSET, Unset

T = TypeVar("T", bound="ManifestArrayConfig")


@attr.s(auto_attribs=True, repr=False)
class ManifestArrayConfig:
    """  """

    _element_definition: List[
        Union[
            SchemaDependency,
            EntitySchemaDependency,
            WorkflowTaskSchemaDependency,
            DropdownDependency,
            ResourceDependency,
            ManifestScalarConfig,
            UnknownType,
        ]
    ]
    _type: ManifestArrayConfigType
    _name: str
    _default_element_name: Union[Unset, str] = UNSET
    _description: Union[Unset, None, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("element_definition={}".format(repr(self._element_definition)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("default_element_name={}".format(repr(self._default_element_name)))
        fields.append("description={}".format(repr(self._description)))
        return "ManifestArrayConfig({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        element_definition = []
        for element_definition_item_data in self._element_definition:
            if isinstance(element_definition_item_data, UnknownType):
                element_definition_item = element_definition_item_data.value
            elif isinstance(element_definition_item_data, SchemaDependency):
                element_definition_item = element_definition_item_data.to_dict()

            elif isinstance(element_definition_item_data, EntitySchemaDependency):
                element_definition_item = element_definition_item_data.to_dict()

            elif isinstance(element_definition_item_data, WorkflowTaskSchemaDependency):
                element_definition_item = element_definition_item_data.to_dict()

            elif isinstance(element_definition_item_data, DropdownDependency):
                element_definition_item = element_definition_item_data.to_dict()

            elif isinstance(element_definition_item_data, ResourceDependency):
                element_definition_item = element_definition_item_data.to_dict()

            else:
                element_definition_item = element_definition_item_data.to_dict()

            element_definition.append(element_definition_item)

        type = self._type.value

        name = self._name
        default_element_name = self._default_element_name
        description = self._description

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if element_definition is not UNSET:
            field_dict["elementDefinition"] = element_definition
        if type is not UNSET:
            field_dict["type"] = type
        if name is not UNSET:
            field_dict["name"] = name
        if default_element_name is not UNSET:
            field_dict["defaultElementName"] = default_element_name
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_element_definition() -> List[
            Union[
                SchemaDependency,
                EntitySchemaDependency,
                WorkflowTaskSchemaDependency,
                DropdownDependency,
                ResourceDependency,
                ManifestScalarConfig,
                UnknownType,
            ]
        ]:
            element_definition = []
            _element_definition = d.pop("elementDefinition")
            for element_definition_item_data in _element_definition:

                def _parse_element_definition_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    SchemaDependency,
                    EntitySchemaDependency,
                    WorkflowTaskSchemaDependency,
                    DropdownDependency,
                    ResourceDependency,
                    ManifestScalarConfig,
                    UnknownType,
                ]:
                    element_definition_item: Union[
                        SchemaDependency,
                        EntitySchemaDependency,
                        WorkflowTaskSchemaDependency,
                        DropdownDependency,
                        ResourceDependency,
                        ManifestScalarConfig,
                        UnknownType,
                    ]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        element_definition_item = SchemaDependency.from_dict(data, strict=True)

                        return element_definition_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        element_definition_item = EntitySchemaDependency.from_dict(data, strict=True)

                        return element_definition_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        element_definition_item = WorkflowTaskSchemaDependency.from_dict(data, strict=True)

                        return element_definition_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        element_definition_item = DropdownDependency.from_dict(data, strict=True)

                        return element_definition_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        element_definition_item = ResourceDependency.from_dict(data, strict=True)

                        return element_definition_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        element_definition_item = ManifestScalarConfig.from_dict(data, strict=True)

                        return element_definition_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                element_definition_item = _parse_element_definition_item(element_definition_item_data)

                element_definition.append(element_definition_item)

            return element_definition

        try:
            element_definition = get_element_definition()
        except KeyError:
            if strict:
                raise
            element_definition = cast(
                List[
                    Union[
                        SchemaDependency,
                        EntitySchemaDependency,
                        WorkflowTaskSchemaDependency,
                        DropdownDependency,
                        ResourceDependency,
                        ManifestScalarConfig,
                        UnknownType,
                    ]
                ],
                UNSET,
            )

        def get_type() -> ManifestArrayConfigType:
            _type = d.pop("type")
            try:
                type = ManifestArrayConfigType(_type)
            except ValueError:
                type = ManifestArrayConfigType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(ManifestArrayConfigType, UNSET)

        def get_name() -> str:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(str, UNSET)

        def get_default_element_name() -> Union[Unset, str]:
            default_element_name = d.pop("defaultElementName")
            return default_element_name

        try:
            default_element_name = get_default_element_name()
        except KeyError:
            if strict:
                raise
            default_element_name = cast(Union[Unset, str], UNSET)

        def get_description() -> Union[Unset, None, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, None, str], UNSET)

        manifest_array_config = cls(
            element_definition=element_definition,
            type=type,
            name=name,
            default_element_name=default_element_name,
            description=description,
        )

        return manifest_array_config

    @property
    def element_definition(
        self,
    ) -> List[
        Union[
            SchemaDependency,
            EntitySchemaDependency,
            WorkflowTaskSchemaDependency,
            DropdownDependency,
            ResourceDependency,
            ManifestScalarConfig,
            UnknownType,
        ]
    ]:
        if isinstance(self._element_definition, Unset):
            raise NotPresentError(self, "element_definition")
        return self._element_definition

    @element_definition.setter
    def element_definition(
        self,
        value: List[
            Union[
                SchemaDependency,
                EntitySchemaDependency,
                WorkflowTaskSchemaDependency,
                DropdownDependency,
                ResourceDependency,
                ManifestScalarConfig,
                UnknownType,
            ]
        ],
    ) -> None:
        self._element_definition = value

    @property
    def type(self) -> ManifestArrayConfigType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: ManifestArrayConfigType) -> None:
        self._type = value

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @property
    def default_element_name(self) -> str:
        if isinstance(self._default_element_name, Unset):
            raise NotPresentError(self, "default_element_name")
        return self._default_element_name

    @default_element_name.setter
    def default_element_name(self, value: str) -> None:
        self._default_element_name = value

    @default_element_name.deleter
    def default_element_name(self) -> None:
        self._default_element_name = UNSET

    @property
    def description(self) -> Optional[str]:
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: Optional[str]) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.custom_entity import CustomEntity
from ..types import UNSET, Unset

T = TypeVar("T", bound="BulkCreateCustomEntitiesAsyncTaskResponse")


@attr.s(auto_attribs=True, repr=False)
class BulkCreateCustomEntitiesAsyncTaskResponse:
    """  """

    _custom_entities: Union[Unset, List[CustomEntity]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("custom_entities={}".format(repr(self._custom_entities)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BulkCreateCustomEntitiesAsyncTaskResponse({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        custom_entities: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._custom_entities, Unset):
            custom_entities = []
            for custom_entities_item_data in self._custom_entities:
                custom_entities_item = custom_entities_item_data.to_dict()

                custom_entities.append(custom_entities_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if custom_entities is not UNSET:
            field_dict["customEntities"] = custom_entities

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_custom_entities() -> Union[Unset, List[CustomEntity]]:
            custom_entities = []
            _custom_entities = d.pop("customEntities")
            for custom_entities_item_data in _custom_entities or []:
                custom_entities_item = CustomEntity.from_dict(custom_entities_item_data, strict=False)

                custom_entities.append(custom_entities_item)

            return custom_entities

        try:
            custom_entities = get_custom_entities()
        except KeyError:
            if strict:
                raise
            custom_entities = cast(Union[Unset, List[CustomEntity]], UNSET)

        bulk_create_custom_entities_async_task_response = cls(
            custom_entities=custom_entities,
        )

        bulk_create_custom_entities_async_task_response.additional_properties = d
        return bulk_create_custom_entities_async_task_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def custom_entities(self) -> List[CustomEntity]:
        if isinstance(self._custom_entities, Unset):
            raise NotPresentError(self, "custom_entities")
        return self._custom_entities

    @custom_entities.setter
    def custom_entities(self, value: List[CustomEntity]) -> None:
        self._custom_entities = value

    @custom_entities.deleter
    def custom_entities(self) -> None:
        self._custom_entities = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AccessPolicy")


@attr.s(auto_attribs=True, repr=False)
class AccessPolicy:
    """  """

    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        return "AccessPolicy({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = self._id
        name = self._name

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        access_policy = cls(
            id=id,
            name=name,
        )

        return access_policy

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.feature_bulk_create import FeatureBulkCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="FeaturesBulkCreateRequest")


@attr.s(auto_attribs=True, repr=False)
class FeaturesBulkCreateRequest:
    """ Inputs for bulk creating a new feature """

    _features: Union[Unset, List[FeatureBulkCreate]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("features={}".format(repr(self._features)))
        return "FeaturesBulkCreateRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        features: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._features, Unset):
            features = []
            for features_item_data in self._features:
                features_item = features_item_data.to_dict()

                features.append(features_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if features is not UNSET:
            field_dict["features"] = features

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_features() -> Union[Unset, List[FeatureBulkCreate]]:
            features = []
            _features = d.pop("features")
            for features_item_data in _features or []:
                features_item = FeatureBulkCreate.from_dict(features_item_data, strict=False)

                features.append(features_item)

            return features

        try:
            features = get_features()
        except KeyError:
            if strict:
                raise
            features = cast(Union[Unset, List[FeatureBulkCreate]], UNSET)

        features_bulk_create_request = cls(
            features=features,
        )

        return features_bulk_create_request

    @property
    def features(self) -> List[FeatureBulkCreate]:
        if isinstance(self._features, Unset):
            raise NotPresentError(self, "features")
        return self._features

    @features.setter
    def features(self, value: List[FeatureBulkCreate]) -> None:
        self._features = value

    @features.deleter
    def features(self) -> None:
        self._features = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class ListEntriesReviewStatus(Enums.KnownString):
    IN_PROGRESS = "IN_PROGRESS"
    ACCEPTED = "ACCEPTED"
    REJECTED = "REJECTED"
    NEEDS_REVIEW = "NEEDS_REVIEW"
    RETRACTED = "RETRACTED"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "ListEntriesReviewStatus":
        if not isinstance(val, str):
            raise ValueError(f"Value of ListEntriesReviewStatus must be a string (encountered: {val})")
        newcls = Enum("ListEntriesReviewStatus", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(ListEntriesReviewStatus, getattr(newcls, "_UNKNOWN"))



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class ListRNAOligosHelmSort(Enums.KnownString):
    MODIFIEDAT = "modifiedAt"
    MODIFIEDATASC = "modifiedAt:asc"
    MODIFIEDATDESC = "modifiedAt:desc"
    NAME = "name"
    NAMEASC = "name:asc"
    NAMEDESC = "name:desc"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "ListRNAOligosHelmSort":
        if not isinstance(val, str):
            raise ValueError(f"Value of ListRNAOligosHelmSort must be a string (encountered: {val})")
        newcls = Enum("ListRNAOligosHelmSort", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(ListRNAOligosHelmSort, getattr(newcls, "_UNKNOWN"))



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class SimpleFieldDefinitionType(Enums.KnownString):
    DNA_SEQUENCE_LINK = "dna_sequence_link"
    AA_SEQUENCE_LINK = "aa_sequence_link"
    CUSTOM_ENTITY_LINK = "custom_entity_link"
    MIXTURE_LINK = "mixture_link"
    BLOB_LINK = "blob_link"
    TEXT = "text"
    LONG_TEXT = "long_text"
    BOOLEAN = "boolean"
    DATETIME = "datetime"
    DATE = "date"
    JSON = "json"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "SimpleFieldDefinitionType":
        if not isinstance(val, str):
            raise ValueError(f"Value of SimpleFieldDefinitionType must be a string (encountered: {val})")
        newcls = Enum("SimpleFieldDefinitionType", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(SimpleFieldDefinitionType, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.bad_request_error_bulk_error import BadRequestErrorBulkError
from ..types import UNSET, Unset

T = TypeVar("T", bound="BadRequestErrorBulk")


@attr.s(auto_attribs=True, repr=False)
class BadRequestErrorBulk:
    """  """

    _error: Union[Unset, BadRequestErrorBulkError] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("error={}".format(repr(self._error)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BadRequestErrorBulk({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        error: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._error, Unset):
            error = self._error.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_error() -> Union[Unset, BadRequestErrorBulkError]:
            error: Union[Unset, Union[Unset, BadRequestErrorBulkError]] = UNSET
            _error = d.pop("error")

            if not isinstance(_error, Unset):
                error = BadRequestErrorBulkError.from_dict(_error)

            return error

        try:
            error = get_error()
        except KeyError:
            if strict:
                raise
            error = cast(Union[Unset, BadRequestErrorBulkError], UNSET)

        bad_request_error_bulk = cls(
            error=error,
        )

        bad_request_error_bulk.additional_properties = d
        return bad_request_error_bulk

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def error(self) -> BadRequestErrorBulkError:
        if isinstance(self._error, Unset):
            raise NotPresentError(self, "error")
        return self._error

    @error.setter
    def error(self, value: BadRequestErrorBulkError) -> None:
        self._error = value

    @error.deleter
    def error(self) -> None:
        self._error = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.string_select_form_field_form_factor import StringSelectFormFieldFormFactor
from ..models.string_select_form_field_option import StringSelectFormFieldOption
from ..models.string_select_form_field_type import StringSelectFormFieldType
from ..types import UNSET, Unset

T = TypeVar("T", bound="StringSelectFormField")


@attr.s(auto_attribs=True, repr=False)
class StringSelectFormField:
    """  """

    _form_factor: Union[Unset, StringSelectFormFieldFormFactor] = UNSET
    _max_selections: Union[Unset, int] = UNSET
    _options: Union[Unset, List[StringSelectFormFieldOption]] = UNSET
    _type: Union[Unset, StringSelectFormFieldType] = UNSET
    _description: Union[Unset, str] = UNSET
    _is_required: Union[Unset, bool] = UNSET
    _key: Union[Unset, str] = UNSET
    _label: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("form_factor={}".format(repr(self._form_factor)))
        fields.append("max_selections={}".format(repr(self._max_selections)))
        fields.append("options={}".format(repr(self._options)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("is_required={}".format(repr(self._is_required)))
        fields.append("key={}".format(repr(self._key)))
        fields.append("label={}".format(repr(self._label)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "StringSelectFormField({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        form_factor: Union[Unset, int] = UNSET
        if not isinstance(self._form_factor, Unset):
            form_factor = self._form_factor.value

        max_selections = self._max_selections
        options: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._options, Unset):
            options = []
            for options_item_data in self._options:
                options_item = options_item_data.to_dict()

                options.append(options_item)

        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        description = self._description
        is_required = self._is_required
        key = self._key
        label = self._label

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if form_factor is not UNSET:
            field_dict["formFactor"] = form_factor
        if max_selections is not UNSET:
            field_dict["maxSelections"] = max_selections
        if options is not UNSET:
            field_dict["options"] = options
        if type is not UNSET:
            field_dict["type"] = type
        if description is not UNSET:
            field_dict["description"] = description
        if is_required is not UNSET:
            field_dict["isRequired"] = is_required
        if key is not UNSET:
            field_dict["key"] = key
        if label is not UNSET:
            field_dict["label"] = label

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_form_factor() -> Union[Unset, StringSelectFormFieldFormFactor]:
            form_factor = UNSET
            _form_factor = d.pop("formFactor")
            if _form_factor is not None and _form_factor is not UNSET:
                try:
                    form_factor = StringSelectFormFieldFormFactor(_form_factor)
                except ValueError:
                    form_factor = StringSelectFormFieldFormFactor.of_unknown(_form_factor)

            return form_factor

        try:
            form_factor = get_form_factor()
        except KeyError:
            if strict:
                raise
            form_factor = cast(Union[Unset, StringSelectFormFieldFormFactor], UNSET)

        def get_max_selections() -> Union[Unset, int]:
            max_selections = d.pop("maxSelections")
            return max_selections

        try:
            max_selections = get_max_selections()
        except KeyError:
            if strict:
                raise
            max_selections = cast(Union[Unset, int], UNSET)

        def get_options() -> Union[Unset, List[StringSelectFormFieldOption]]:
            options = []
            _options = d.pop("options")
            for options_item_data in _options or []:
                options_item = StringSelectFormFieldOption.from_dict(options_item_data, strict=False)

                options.append(options_item)

            return options

        try:
            options = get_options()
        except KeyError:
            if strict:
                raise
            options = cast(Union[Unset, List[StringSelectFormFieldOption]], UNSET)

        def get_type() -> Union[Unset, StringSelectFormFieldType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = StringSelectFormFieldType(_type)
                except ValueError:
                    type = StringSelectFormFieldType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, StringSelectFormFieldType], UNSET)

        def get_description() -> Union[Unset, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, str], UNSET)

        def get_is_required() -> Union[Unset, bool]:
            is_required = d.pop("isRequired")
            return is_required

        try:
            is_required = get_is_required()
        except KeyError:
            if strict:
                raise
            is_required = cast(Union[Unset, bool], UNSET)

        def get_key() -> Union[Unset, str]:
            key = d.pop("key")
            return key

        try:
            key = get_key()
        except KeyError:
            if strict:
                raise
            key = cast(Union[Unset, str], UNSET)

        def get_label() -> Union[Unset, str]:
            label = d.pop("label")
            return label

        try:
            label = get_label()
        except KeyError:
            if strict:
                raise
            label = cast(Union[Unset, str], UNSET)

        string_select_form_field = cls(
            form_factor=form_factor,
            max_selections=max_selections,
            options=options,
            type=type,
            description=description,
            is_required=is_required,
            key=key,
            label=label,
        )

        string_select_form_field.additional_properties = d
        return string_select_form_field

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def form_factor(self) -> StringSelectFormFieldFormFactor:
        if isinstance(self._form_factor, Unset):
            raise NotPresentError(self, "form_factor")
        return self._form_factor

    @form_factor.setter
    def form_factor(self, value: StringSelectFormFieldFormFactor) -> None:
        self._form_factor = value

    @form_factor.deleter
    def form_factor(self) -> None:
        self._form_factor = UNSET

    @property
    def max_selections(self) -> int:
        if isinstance(self._max_selections, Unset):
            raise NotPresentError(self, "max_selections")
        return self._max_selections

    @max_selections.setter
    def max_selections(self, value: int) -> None:
        self._max_selections = value

    @max_selections.deleter
    def max_selections(self) -> None:
        self._max_selections = UNSET

    @property
    def options(self) -> List[StringSelectFormFieldOption]:
        if isinstance(self._options, Unset):
            raise NotPresentError(self, "options")
        return self._options

    @options.setter
    def options(self, value: List[StringSelectFormFieldOption]) -> None:
        self._options = value

    @options.deleter
    def options(self) -> None:
        self._options = UNSET

    @property
    def type(self) -> StringSelectFormFieldType:
        """The type of this form field. Type declares how this field behaves and dictates the additional properties passed along with the required properties like label and key"""
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: StringSelectFormFieldType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def description(self) -> str:
        """ Description of the purpose of this field """
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: str) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def is_required(self) -> bool:
        """ Whether this field is required to be filled out in order to be a valid submission """
        if isinstance(self._is_required, Unset):
            raise NotPresentError(self, "is_required")
        return self._is_required

    @is_required.setter
    def is_required(self, value: bool) -> None:
        self._is_required = value

    @is_required.deleter
    def is_required(self) -> None:
        self._is_required = UNSET

    @property
    def key(self) -> str:
        """ Reference key of this form field. Used to fix identity of fields beyond the label """
        if isinstance(self._key, Unset):
            raise NotPresentError(self, "key")
        return self._key

    @key.setter
    def key(self, value: str) -> None:
        self._key = value

    @key.deleter
    def key(self) -> None:
        self._key = UNSET

    @property
    def label(self) -> str:
        """ End user facing name of this form field. What you see when you fill out the form each time """
        if isinstance(self._label, Unset):
            raise NotPresentError(self, "label")
        return self._label

    @label.setter
    def label(self, value: str) -> None:
        self._label = value

    @label.deleter
    def label(self) -> None:
        self._label = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.entry_template import EntryTemplate
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryTemplatesPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class EntryTemplatesPaginatedList:
    """  """

    _entry_templates: Union[Unset, List[EntryTemplate]] = UNSET
    _next_token: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("entry_templates={}".format(repr(self._entry_templates)))
        fields.append("next_token={}".format(repr(self._next_token)))
        return "EntryTemplatesPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        entry_templates: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._entry_templates, Unset):
            entry_templates = []
            for entry_templates_item_data in self._entry_templates:
                entry_templates_item = entry_templates_item_data.to_dict()

                entry_templates.append(entry_templates_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if entry_templates is not UNSET:
            field_dict["entryTemplates"] = entry_templates
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_entry_templates() -> Union[Unset, List[EntryTemplate]]:
            entry_templates = []
            _entry_templates = d.pop("entryTemplates")
            for entry_templates_item_data in _entry_templates or []:
                entry_templates_item = EntryTemplate.from_dict(entry_templates_item_data, strict=False)

                entry_templates.append(entry_templates_item)

            return entry_templates

        try:
            entry_templates = get_entry_templates()
        except KeyError:
            if strict:
                raise
            entry_templates = cast(Union[Unset, List[EntryTemplate]], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        entry_templates_paginated_list = cls(
            entry_templates=entry_templates,
            next_token=next_token,
        )

        return entry_templates_paginated_list

    @property
    def entry_templates(self) -> List[EntryTemplate]:
        if isinstance(self._entry_templates, Unset):
            raise NotPresentError(self, "entry_templates")
        return self._entry_templates

    @entry_templates.setter
    def entry_templates(self, value: List[EntryTemplate]) -> None:
        self._entry_templates = value

    @entry_templates.deleter
    def entry_templates(self) -> None:
        self._entry_templates = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class WebhookEnvelopeVersion(Enums.KnownString):
    VALUE_0 = "0"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "WebhookEnvelopeVersion":
        if not isinstance(val, str):
            raise ValueError(f"Value of WebhookEnvelopeVersion must be a string (encountered: {val})")
        newcls = Enum("WebhookEnvelopeVersion", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(WebhookEnvelopeVersion, getattr(newcls, "_UNKNOWN"))



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.not_found_error import NotFoundError
from ...models.workflow_task_schemas_paginated_list import WorkflowTaskSchemasPaginatedList
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    modified_at: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/workflow-task-schemas".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    params: Dict[str, Any] = {}
    if not isinstance(next_token, Unset) and next_token is not None:
        params["nextToken"] = next_token
    if not isinstance(page_size, Unset) and page_size is not None:
        params["pageSize"] = page_size
    if not isinstance(modified_at, Unset) and modified_at is not None:
        params["modifiedAt"] = modified_at

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[WorkflowTaskSchemasPaginatedList, NotFoundError]]:
    if response.status_code == 200:
        response_200 = WorkflowTaskSchemasPaginatedList.from_dict(response.json(), strict=False)

        return response_200
    if response.status_code == 404:
        response_404 = NotFoundError.from_dict(response.json(), strict=False)

        return response_404
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[WorkflowTaskSchemasPaginatedList, NotFoundError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    modified_at: Union[Unset, str] = UNSET,
) -> Response[Union[WorkflowTaskSchemasPaginatedList, NotFoundError]]:
    kwargs = _get_kwargs(
        client=client,
        next_token=next_token,
        page_size=page_size,
        modified_at=modified_at,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    modified_at: Union[Unset, str] = UNSET,
) -> Optional[Union[WorkflowTaskSchemasPaginatedList, NotFoundError]]:
    """ List workflow task schemas """

    return sync_detailed(
        client=client,
        next_token=next_token,
        page_size=page_size,
        modified_at=modified_at,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    modified_at: Union[Unset, str] = UNSET,
) -> Response[Union[WorkflowTaskSchemasPaginatedList, NotFoundError]]:
    kwargs = _get_kwargs(
        client=client,
        next_token=next_token,
        page_size=page_size,
        modified_at=modified_at,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    modified_at: Union[Unset, str] = UNSET,
) -> Optional[Union[WorkflowTaskSchemasPaginatedList, NotFoundError]]:
    """ List workflow task schemas """

    return (
        await asyncio_detailed(
            client=client,
            next_token=next_token,
            page_size=page_size,
            modified_at=modified_at,
        )
    ).parsed



from typing import Union

from ..extensions import UnknownType
from ..models.analysis_step_update_with_message import AnalysisStepUpdateWithMessage
from ..models.analysis_step_update_without_message import AnalysisStepUpdateWithoutMessage

AnalysisStepUpdate = Union[AnalysisStepUpdateWithoutMessage, AnalysisStepUpdateWithMessage, UnknownType]



import datetime
from typing import Any, cast, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.form_definition_version import FormDefinitionVersion
from ..types import UNSET, Unset

T = TypeVar("T", bound="FormDefinition")


@attr.s(auto_attribs=True, repr=False)
class FormDefinition:
    """  """

    _id: Union[Unset, str] = UNSET
    _latest_version: Union[Unset, FormDefinitionVersion] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _name: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("latest_version={}".format(repr(self._latest_version)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("name={}".format(repr(self._name)))
        return "FormDefinition({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = self._id
        latest_version: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._latest_version, Unset):
            latest_version = self._latest_version.to_dict()

        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        name = self._name

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if latest_version is not UNSET:
            field_dict["latestVersion"] = latest_version
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_latest_version() -> Union[Unset, FormDefinitionVersion]:
            latest_version: Union[Unset, Union[Unset, FormDefinitionVersion]] = UNSET
            _latest_version = d.pop("latestVersion")

            if not isinstance(_latest_version, Unset):
                latest_version = FormDefinitionVersion.from_dict(_latest_version)

            return latest_version

        try:
            latest_version = get_latest_version()
        except KeyError:
            if strict:
                raise
            latest_version = cast(Union[Unset, FormDefinitionVersion], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        form_definition = cls(
            id=id,
            latest_version=latest_version,
            modified_at=modified_at,
            name=name,
        )

        return form_definition

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def latest_version(self) -> FormDefinitionVersion:
        if isinstance(self._latest_version, Unset):
            raise NotPresentError(self, "latest_version")
        return self._latest_version

    @latest_version.setter
    def latest_version(self, value: FormDefinitionVersion) -> None:
        self._latest_version = value

    @latest_version.deleter
    def latest_version(self) -> None:
        self._latest_version = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ Time when the definition was last modified. Will be updated when a new version becomes the default """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.access_policy import AccessPolicy
from ..models.app_collaborator import AppCollaborator
from ..models.org_membership_collaborator import OrgMembershipCollaborator
from ..models.team_membership_collaborator import TeamMembershipCollaborator
from ..models.user_collaborator import UserCollaborator
from ..types import UNSET, Unset

T = TypeVar("T", bound="Collaboration")


@attr.s(auto_attribs=True, repr=False)
class Collaboration:
    """  """

    _access_policy: Union[Unset, AccessPolicy] = UNSET
    _collaborator: Union[
        Unset,
        OrgMembershipCollaborator,
        TeamMembershipCollaborator,
        UserCollaborator,
        AppCollaborator,
        UnknownType,
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("access_policy={}".format(repr(self._access_policy)))
        fields.append("collaborator={}".format(repr(self._collaborator)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "Collaboration({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        access_policy: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._access_policy, Unset):
            access_policy = self._access_policy.to_dict()

        collaborator: Union[Unset, Dict[str, Any]]
        if isinstance(self._collaborator, Unset):
            collaborator = UNSET
        elif isinstance(self._collaborator, UnknownType):
            collaborator = self._collaborator.value
        elif isinstance(self._collaborator, OrgMembershipCollaborator):
            collaborator = UNSET
            if not isinstance(self._collaborator, Unset):
                collaborator = self._collaborator.to_dict()

        elif isinstance(self._collaborator, TeamMembershipCollaborator):
            collaborator = UNSET
            if not isinstance(self._collaborator, Unset):
                collaborator = self._collaborator.to_dict()

        elif isinstance(self._collaborator, UserCollaborator):
            collaborator = UNSET
            if not isinstance(self._collaborator, Unset):
                collaborator = self._collaborator.to_dict()

        else:
            collaborator = UNSET
            if not isinstance(self._collaborator, Unset):
                collaborator = self._collaborator.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if access_policy is not UNSET:
            field_dict["accessPolicy"] = access_policy
        if collaborator is not UNSET:
            field_dict["collaborator"] = collaborator

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_access_policy() -> Union[Unset, AccessPolicy]:
            access_policy: Union[Unset, Union[Unset, AccessPolicy]] = UNSET
            _access_policy = d.pop("accessPolicy")

            if not isinstance(_access_policy, Unset):
                access_policy = AccessPolicy.from_dict(_access_policy)

            return access_policy

        try:
            access_policy = get_access_policy()
        except KeyError:
            if strict:
                raise
            access_policy = cast(Union[Unset, AccessPolicy], UNSET)

        def get_collaborator() -> Union[
            Unset,
            OrgMembershipCollaborator,
            TeamMembershipCollaborator,
            UserCollaborator,
            AppCollaborator,
            UnknownType,
        ]:
            collaborator: Union[
                Unset,
                OrgMembershipCollaborator,
                TeamMembershipCollaborator,
                UserCollaborator,
                AppCollaborator,
                UnknownType,
            ]
            _collaborator = d.pop("collaborator")

            if not isinstance(_collaborator, Unset):
                discriminator = _collaborator["type"]
                if discriminator == "APP":
                    collaborator = AppCollaborator.from_dict(_collaborator)
                elif discriminator == "ORGANIZATION_MEMBER":
                    collaborator = OrgMembershipCollaborator.from_dict(_collaborator)
                elif discriminator == "TEAM_MEMBER":
                    collaborator = TeamMembershipCollaborator.from_dict(_collaborator)
                elif discriminator == "USER":
                    collaborator = UserCollaborator.from_dict(_collaborator)
                else:
                    collaborator = UnknownType(value=_collaborator)

            return collaborator

        try:
            collaborator = get_collaborator()
        except KeyError:
            if strict:
                raise
            collaborator = cast(
                Union[
                    Unset,
                    OrgMembershipCollaborator,
                    TeamMembershipCollaborator,
                    UserCollaborator,
                    AppCollaborator,
                    UnknownType,
                ],
                UNSET,
            )

        collaboration = cls(
            access_policy=access_policy,
            collaborator=collaborator,
        )

        collaboration.additional_properties = d
        return collaboration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def access_policy(self) -> AccessPolicy:
        if isinstance(self._access_policy, Unset):
            raise NotPresentError(self, "access_policy")
        return self._access_policy

    @access_policy.setter
    def access_policy(self, value: AccessPolicy) -> None:
        self._access_policy = value

    @access_policy.deleter
    def access_policy(self) -> None:
        self._access_policy = UNSET

    @property
    def collaborator(
        self,
    ) -> Union[
        OrgMembershipCollaborator, TeamMembershipCollaborator, UserCollaborator, AppCollaborator, UnknownType
    ]:
        if isinstance(self._collaborator, Unset):
            raise NotPresentError(self, "collaborator")
        return self._collaborator

    @collaborator.setter
    def collaborator(
        self,
        value: Union[
            OrgMembershipCollaborator,
            TeamMembershipCollaborator,
            UserCollaborator,
            AppCollaborator,
            UnknownType,
        ],
    ) -> None:
        self._collaborator = value

    @collaborator.deleter
    def collaborator(self) -> None:
        self._collaborator = UNSET



import datetime
from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.worklist_creator import WorklistCreator
from ..models.worklist_item_summary import WorklistItemSummary
from ..models.worklist_type import WorklistType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Worklist")


@attr.s(auto_attribs=True, repr=False)
class Worklist:
    """A worklist and its items as summaries."""

    _api_url: Union[Unset, str] = UNSET
    _created_at: Union[Unset, datetime.datetime] = UNSET
    _creator: Union[Unset, WorklistCreator] = UNSET
    _id: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _name: Union[Unset, str] = UNSET
    _type: Union[Unset, WorklistType] = UNSET
    _worklist_items: Union[Unset, List[WorklistItemSummary]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("api_url={}".format(repr(self._api_url)))
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("creator={}".format(repr(self._creator)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("worklist_items={}".format(repr(self._worklist_items)))
        return "Worklist({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        api_url = self._api_url
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creator, Unset):
            creator = self._creator.to_dict()

        id = self._id
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        name = self._name
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        worklist_items: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._worklist_items, Unset):
            worklist_items = []
            for worklist_items_item_data in self._worklist_items:
                worklist_items_item = worklist_items_item_data.to_dict()

                worklist_items.append(worklist_items_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if api_url is not UNSET:
            field_dict["apiURL"] = api_url
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if creator is not UNSET:
            field_dict["creator"] = creator
        if id is not UNSET:
            field_dict["id"] = id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if name is not UNSET:
            field_dict["name"] = name
        if type is not UNSET:
            field_dict["type"] = type
        if worklist_items is not UNSET:
            field_dict["worklistItems"] = worklist_items

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_api_url() -> Union[Unset, str]:
            api_url = d.pop("apiURL")
            return api_url

        try:
            api_url = get_api_url()
        except KeyError:
            if strict:
                raise
            api_url = cast(Union[Unset, str], UNSET)

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_creator() -> Union[Unset, WorklistCreator]:
            creator: Union[Unset, Union[Unset, WorklistCreator]] = UNSET
            _creator = d.pop("creator")

            if not isinstance(_creator, Unset):
                creator = WorklistCreator.from_dict(_creator)

            return creator

        try:
            creator = get_creator()
        except KeyError:
            if strict:
                raise
            creator = cast(Union[Unset, WorklistCreator], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_type() -> Union[Unset, WorklistType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = WorklistType(_type)
                except ValueError:
                    type = WorklistType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, WorklistType], UNSET)

        def get_worklist_items() -> Union[Unset, List[WorklistItemSummary]]:
            worklist_items = []
            _worklist_items = d.pop("worklistItems")
            for worklist_items_item_data in _worklist_items or []:
                worklist_items_item = WorklistItemSummary.from_dict(worklist_items_item_data, strict=False)

                worklist_items.append(worklist_items_item)

            return worklist_items

        try:
            worklist_items = get_worklist_items()
        except KeyError:
            if strict:
                raise
            worklist_items = cast(Union[Unset, List[WorklistItemSummary]], UNSET)

        worklist = cls(
            api_url=api_url,
            created_at=created_at,
            creator=creator,
            id=id,
            modified_at=modified_at,
            name=name,
            type=type,
            worklist_items=worklist_items,
        )

        return worklist

    @property
    def api_url(self) -> str:
        """ The canonical url of the Worklist in the API. """
        if isinstance(self._api_url, Unset):
            raise NotPresentError(self, "api_url")
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = value

    @api_url.deleter
    def api_url(self) -> None:
        self._api_url = UNSET

    @property
    def created_at(self) -> datetime.datetime:
        """ Date and time the worklist was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def creator(self) -> WorklistCreator:
        if isinstance(self._creator, Unset):
            raise NotPresentError(self, "creator")
        return self._creator

    @creator.setter
    def creator(self, value: WorklistCreator) -> None:
        self._creator = value

    @creator.deleter
    def creator(self) -> None:
        self._creator = UNSET

    @property
    def id(self) -> str:
        """ ID of the worklist """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ Date and time the worklist was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def name(self) -> str:
        """ Name of the worklist """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def type(self) -> WorklistType:
        """The type of items a worklist contains."""
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: WorklistType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def worklist_items(self) -> List[WorklistItemSummary]:
        if isinstance(self._worklist_items, Unset):
            raise NotPresentError(self, "worklist_items")
        return self._worklist_items

    @worklist_items.setter
    def worklist_items(self, value: List[WorklistItemSummary]) -> None:
        self._worklist_items = value

    @worklist_items.deleter
    def worklist_items(self) -> None:
        self._worklist_items = UNSET
